@@ -1,4 +1,4 @@
-/* $Id: acconfig.h,v 1.5 2000/08/07 02:16:24 astyanax Exp $ */
+/* $Id: acconfig.h,v 1.8 2000/12/01 18:46:01 astyanax Exp $ */
 
 /* Define this if you have the wresize function in your ncurses-type library */
 #undef HAVE_WRESIZE
@@ -24,3 +24,16 @@
 /* Define to use the slang wrappers for curses instead of native curses */
 #undef USE_SLANG
 
+/* Define this to enable the extra stuff */
+#undef NANO_EXTRA
+
+/* Define to disable the tab completion code Chris worked so hard on! */
+#undef DISABLE_TABCOMP
+
+/* Define this to disable the justify routine */
+#undef DISABLE_JUSTIFY
+
+/* Define this to disable the use(full|less) spelling functions */
+#undef DISABLE_SPELL
+
+
