@@ -1,4 +1,4 @@
-/* $Id: files.c,v 1.31 2000/11/18 03:05:50 astyanax Exp $ */
+/* $Id: files.c,v 1.44 2000/12/03 03:01:12 astyanax Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -179,7 +179,7 @@ int read_file(int fd, char *filename)
 	buf[0] = 0;
     }
     /* Did we even GET a file? */
-    if (totsize == 0) {
+    if (totsize == 0 || fileptr == NULL) {
 	new_file();
 	statusbar(_("Read %d lines"), num_lines);
 	return 1;
@@ -245,6 +245,7 @@ int open_file(char *filename, int insert, int quiet)
 int do_insertfile(void)
 {
     int i;
+    char *realname = NULL;
 
     wrap_reset();
     i = statusq(1, writefile_list, WRITEFILE_LIST_LEN, "",
@@ -255,7 +256,14 @@ int do_insertfile(void)
 	fprintf(stderr, "filename is %s", answer);
 #endif
 
-	i = open_file(answer, 1, 0);
+#ifndef DISABLE_TABCOMP
+        realname = real_dir_from_tilde(answer);
+#else
+        realname = mallocstrcpy(realname, answer);
+#endif
+
+	i = open_file(realname, 1, 0);
+	free(realname);
 
 	dump_buffer(fileage);
 	set_modified();
@@ -285,16 +293,17 @@ int do_insertfile(void)
  * we don't set the global variable filename to it's name, and don't
  * print out how many lines we wrote on the statusbar.
  * 
- * Note that tmp is only set to 1 for storing temporary files internal
- * to the editor, and is completely different from TEMP_OPT.
+ * tmp means we are writing a tmp file in a secute fashion.  We use
+ * it when spell checking or dumping the file on an error.
  */
 int write_file(char *name, int tmp)
 {
     long size, lineswritten = 0;
     char buf[PATH_MAX + 1];
     filestruct *fileptr;
-    int fd, mask = 0;
+    int fd, mask = 0, realexists;
     struct stat st;
+    static char *realname = NULL;
 
     if (!strcmp(name, "")) {
 	statusbar(_("Cancelled"));
@@ -303,16 +312,31 @@ int write_file(char *name, int tmp)
     titlebar();
     fileptr = fileage;
 
+    if (realname != NULL)
+	free(realname);
+
+#ifndef DISABLE_TABCOMP
+    realname = real_dir_from_tilde(name);
+#else
+    realname = mallocstrcpy(realname, name);
+#endif
+
+    /* Save the state of file at the end of the symlink */
+    realexists = stat(realname, &st);
 
     /* Check to see if the file is a regular file and FOLLOW_SYMLINKS is
        set.  If so then don't do the delete and recreate code which would
        cause unexpected behavior */
-    lstat(name, &st);
+    lstat(realname, &st);
 
-    /* Open the file and truncate it.  Trust the symlink. */
-    if ((ISSET(FOLLOW_SYMLINKS) || !S_ISLNK(st.st_mode)) && !tmp) {
+    /* New case: if the file exists, just give up.  Easy way out of
+	all security issues */
+    if (tmp && realexists != -1)
+	 return -1;
+    else if (ISSET(FOLLOW_SYMLINKS) || !S_ISLNK(st.st_mode)) {
 
-	if ((fd = open(name, O_WRONLY | O_CREAT | O_TRUNC,
+	/* Open the file and truncate it.  Trust the symlink. */
+	if ((fd = open(realname, O_WRONLY | O_CREAT | O_TRUNC,
 		       S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH |
 		       S_IWOTH)) == -1) {
 	    if (ISSET(TEMP_OPT)) {
@@ -321,18 +345,19 @@ int write_file(char *name, int tmp)
 	    }
 	    statusbar(_("Could not open file for writing: %s"),
 		      strerror(errno));
+	    free(realname);
 	    return -1;
 	}
     }
     /* Don't follow symlink.  Create new file. */
     else {
-	if (strlen(name) > (PATH_MAX - 7)) {
+	if (strlen(realname) > (PATH_MAX - 7)) {
 	    statusbar(_("Could not open file: Path length exceeded."));
 	    return -1;
 	}
 
 	memset(buf, 0x00, PATH_MAX + 1);
-	strcat(buf, name);
+	strcat(buf, realname);
 	strcat(buf, ".XXXXXX");
 	if ((fd = mkstemp(buf)) == -1) {
 	    if (ISSET(TEMP_OPT)) {
@@ -385,13 +410,13 @@ int write_file(char *name, int tmp)
 
 
     if (close(fd) == -1) {
-	statusbar(_("Could not close %s: %s"), name, strerror(errno));
+	statusbar(_("Could not close %s: %s"), realname, strerror(errno));
 	unlink(buf);
 	return -1;
     }
 
     if (!ISSET(FOLLOW_SYMLINKS) || tmp) {
-	if (stat(name, &st) == -1) {
+	if (realexists == -1) {
 	    /* Use default umask as file permisions if file is a new file. */
 	    mask = umask(0);
 	    umask(mask);
@@ -404,47 +429,51 @@ int write_file(char *name, int tmp)
 	} else {
 	    /* Use permissions from file we are overwriting. */
 	    mask = st.st_mode;
-	    if (unlink(name) == -1) {
+	    if (unlink(realname) == -1) {
 		if (errno != ENOENT) {
 		    statusbar(_("Could not open %s for writing: %s"),
-			      name, strerror(errno));
+			      realname, strerror(errno));
 		    unlink(buf);
 		    return -1;
 		}
 	    }
 	}
 
-	if (link(buf, name) != -1)
-	    unlink(buf);
-	else if (errno != EPERM) {
-	    statusbar(_("Could not open %s for writing: %s"),
+	if (!tmp) {
+	    if (link(buf, realname) != -1)
+		unlink(buf);
+	    else if (errno != EPERM) {
+		statusbar(_("Could not open %s for writing: %s"),
 		      name, strerror(errno));
-	    unlink(buf);
-	    return -1;
-	} else if (rename(buf, name) == -1) {	/* Try a rename?? */
-	    statusbar(_("Could not open %s for writing: %s"),
-		      name, strerror(errno));
-	    unlink(buf);
-	    return -1;
+		unlink(buf);
+		return -1;
+	    } else if (rename(buf, realname) == -1) {	/* Try a rename?? */
+ 		statusbar(_("Could not open %s for writing: %s"),
+		      realname, strerror(errno));
+		unlink(buf);
+		return -1;
+	    }
 	}
-	if (chmod(name, mask) == -1) {
+	if (chmod(realname, mask) == -1)
 	    statusbar(_("Could not set permissions %o on %s: %s"),
-		      mask, name, strerror(errno));
-	}
+		      mask, realname, strerror(errno));
 
     }
     if (!tmp) {
-	strncpy(filename, name, 132);
+	strncpy(filename, realname, 132);
 	statusbar(_("Wrote %d lines"), lineswritten);
+        UNSET(MODIFIED);
+        titlebar();
     }
-    UNSET(MODIFIED);
-    titlebar();
     return 1;
 }
 
 int do_writeout(int exiting)
 {
     int i = 0;
+#ifdef NANO_EXTRA
+    static int did_cred = 0;
+#endif
 
     answer = mallocstrcpy(answer, filename);
 
@@ -471,6 +500,14 @@ int do_writeout(int exiting)
 #ifdef DEBUG
 	    fprintf(stderr, _("filename is %s"), answer);
 #endif
+
+#ifdef NANO_EXTRA
+	    if (exiting && !ISSET(TEMP_OPT) && !strcasecmp(answer, "zzy") && !did_cred) {
+		do_credits();
+		did_cred = 1;
+		return - 1;
+	    }
+#endif
 	    if (strcmp(answer, filename)) {
 		struct stat st;
 		if (!stat(answer, &st)) {
@@ -497,6 +534,110 @@ int do_writeout_void(void)
     return do_writeout(0);
 }
 
+#ifndef DISABLE_TABCOMP
+static char **homedirs;
+
+/* Return a malloc()ed string containing the actual directory, used
+ * to convert ~user and ~/ notation...
+ */
+char *real_dir_from_tilde (char *buf)
+{
+    char *dirtmp = NULL, *line = NULL, byte[1], *lineptr;
+    int fd, i, status, searchctr = 1;
+
+    if (buf[0] == '~') {
+	if (buf[1] == '~')
+	    goto abort;		/* Handle ~~ without segfaulting =) */
+	else if (buf[1] == '/') {
+	    if (getenv("HOME") != NULL) {
+		dirtmp = nmalloc(strlen(buf) + 2 + strlen(getenv("HOME")));
+
+		sprintf(dirtmp, "%s/%s", getenv("HOME"), &buf[2]);
+	    }
+	}
+	else if (buf[1] != 0) {
+
+	    if((fd = open("/etc/passwd", O_RDONLY)) == -1)
+		goto abort;
+
+	    /* Figure how how much of of the str we need to compare */
+	    for (searchctr = 1; buf[searchctr] != '/' &&
+			buf[searchctr] != 0; searchctr++)
+		;
+
+	    do {
+		i = 0;
+		line = nmalloc(1);
+		while ((status = read(fd, byte, 1)) != 0 && byte[0] != '\n') {
+
+		    line[i] = byte[0];
+		    i++;
+		    line = nrealloc(line, i+1);
+		}
+		line[i] = 0;
+
+		if (i == 0)
+		     goto abort;
+
+		line[i] = 0;
+		lineptr = strtok(line, ":");
+
+		if (!strncmp(lineptr, &buf[1], searchctr - 1)) {
+
+		    /* Okay, skip to the password portion now */
+		    for (i = 0; i <= 4 && lineptr != NULL; i++)
+			lineptr = strtok(NULL, ":");
+
+		    if (lineptr == NULL)
+			goto abort;
+
+		     /* Else copy the new string into the new buf */
+		    dirtmp = nmalloc(strlen(buf) + 2 + strlen(lineptr));
+
+		    sprintf(dirtmp, "%s%s", lineptr, &buf[searchctr]);
+		    free(line);
+		    break;
+		}
+	
+		free(line);
+
+	    } while (status != 0);
+	}
+    }
+    else
+	dirtmp = mallocstrcpy(dirtmp, buf);
+
+    return dirtmp;	
+
+abort:		
+    dirtmp = mallocstrcpy(dirtmp, buf);
+    return dirtmp;
+}
+
+/* Tack a slash onto the string we're completing if it's a directory */
+int append_slash_if_dir(char *buf, int *lastWasTab, int *place)
+{
+    char *dirptr;
+    struct stat fileinfo;
+    int ret = 0;
+
+    dirptr = real_dir_from_tilde(buf);
+
+    if (stat(dirptr, &fileinfo) == -1)
+    	ret = 0;
+    else if (S_ISDIR(fileinfo.st_mode)) {
+	strncat(buf, "/", 1);
+	*place += 1;
+	/* now we start over again with # of tabs so far */
+	*lastWasTab = 0;
+	ret = 1;
+    }
+
+    if (dirptr != buf)
+    	free(dirptr);
+
+    return ret;
+}
 
 /*
  * These functions (username_tab_completion, cwd_tab_completion, and
@@ -517,24 +658,90 @@ int do_writeout_void(void)
  * This code may safely be consumed by a BSD or GPL license.
  */
 
-#if 0
 char **username_tab_completion(char *buf, int *num_matches)
 {
-    char **matches = (char **) NULL;
-    *num_matches = 0;
+    char **matches = (char **) NULL, *line = NULL, *lineptr;
+    char *matchline = NULL, *matchdir = NULL;
+	
+    int fd, i = 0, status = 1;
+    char byte[1];
+
+    if((fd = open("/etc/passwd", O_RDONLY)) == -1) {
+	return NULL;
+    }
+
+    if (homedirs != NULL) {
+	for (i = 0; i < *num_matches; i++)
+	    free(homedirs[i]);
+	free(homedirs);
+	homedirs = (char **) NULL;
+	*num_matches = 0;
+    }
+    matches = nmalloc(BUFSIZ);
+    homedirs = nmalloc(BUFSIZ);
+    strcat(buf, "*");
+    do {
+	i = 0;
+	line = nmalloc(1);
+	while ((status = read(fd, byte, 1)) != 0 && byte[0] != '\n') {
+
+	    line[i] = byte[0];
+	    i++;
+	    line = nrealloc(line, i+1);
+	}
+
+	if (i == 0)
+	    break;
+
+	line[i] = 0;
+	lineptr = strtok(line, ":");
+
+	if (check_wildcard_match(line, &buf[1]) == TRUE) {
+
+	    if (*num_matches == BUFSIZ)
+		break;
+
+	    /* Cool, found a match.  Add it to the list
+	     * This makes a lot more sense to me (Chris) this way...
+	     */
+	    matchline = nmalloc(strlen(line) + 2);
+	    sprintf(matchline, "~%s", line);
+
+	    for (i = 0; i <= 4 && lineptr != NULL; i++)
+		lineptr = strtok(NULL, ":");
+
+	    if (lineptr == NULL)
+		break;
+
+	    matchdir = mallocstrcpy(matchdir, lineptr);
+	    homedirs[*num_matches] = matchdir;
+	    matches[*num_matches] = matchline;
+
+	    ++*num_matches;
+
+	    /* If there's no more room, bail out */
+	    if (*num_matches == BUFSIZ)
+		break;
+	}
+
+	free(line);
+
+    } while (status != 0);
+
+    close(fd);
+    return matches;
 #ifdef DEBUG
     fprintf(stderr, "\nin username_tab_completion\n");
 #endif
     return (matches);
 }
-#endif
 
 /* This was originally called exe_n_cwd_tab_completion, but we're not
    worried about executables, only filenames :> */
 
 char **cwd_tab_completion(char *buf, int *num_matches)
 {
-    char *dirName, *tmp = NULL, *tmp2 = NULL;
+    char *dirName, *dirtmp = NULL, *tmp = NULL, *tmp2 = NULL;
     char **matches = (char **) NULL;
     DIR *dir;
     struct dirent *next;
@@ -569,6 +776,17 @@ char **cwd_tab_completion(char *buf, int *num_matches)
     fprintf(stderr, "\ntmp = %s\n", tmp);
 #endif
 
+    dirtmp = real_dir_from_tilde(dirName);
+    free(dirName);
+    dirName = dirtmp;
+
+#ifdef DEBUG
+    fprintf(stderr, "\nDir = %s\n", dirName);
+    fprintf(stderr, "\nbuf = %s\n", buf);
+    fprintf(stderr, "\ntmp = %s\n", tmp);
+#endif
+
+
     dir = opendir(dirName);
     if (!dir) {
 	/* Don't print an error, just shut up and return */
@@ -616,9 +834,8 @@ char *input_tab(char *buf, int place, int *lastWasTab, int *newplace)
     static int num_matches = 0, match_matches = 0;
     static char **matches = (char **) NULL;
     int pos = place, i = 0, col = 0, editline = 0;
-    int longestname = 0;
+    int longestname = 0, is_dir = 0;
     char *foo;
-    struct stat fileinfo;
 
     if (*lastWasTab == FALSE) {
 	char *tmp, *copyto, *matchBuf;
@@ -648,9 +865,9 @@ char *input_tab(char *buf, int place, int *lastWasTab, int *newplace)
 	/* If the word starts with `~' and there is no slash in the word, 
 	 * then try completing this word as a username. */
 
-	/* FIXME -- this check is broken!
+	/* FIXME -- this check is broken! */
 	   if (*tmp == '~' && !strchr(tmp, '/'))
-	   matches = username_tab_completion(tmp, &num_matches); */
+	   matches = username_tab_completion(tmp, &num_matches);
 
 	/* Try to match everything in the current working directory that
 	 * matches.  */
@@ -680,21 +897,11 @@ char *input_tab(char *buf, int place, int *lastWasTab, int *newplace)
 	    } else
 		tmp = buf;
 
-	    if (!strcmp(tmp, matches[0])) {
+	    if (!strcmp(tmp, matches[0]))
+		is_dir = append_slash_if_dir(buf, lastWasTab, newplace);
 
-		/* Is it a directory? */
-		if (stat(buf, &fileinfo) == -1)
-		    break;
-		else if (S_ISDIR(fileinfo.st_mode)) {
-
-		    /* Tack on a slash */
-		    strncat(buf, "/", 1);
-		    *newplace += 1;
-		    /* now we start over with 0 tabs so far */
-		    *lastWasTab = 0;
-		}
+	    if (is_dir)
 		break;
-	    }
 
 	    copyto = tmp;
 	    for (pos = 0; *tmp == matches[0][pos] &&
@@ -705,14 +912,9 @@ char *input_tab(char *buf, int place, int *lastWasTab, int *newplace)
 	    strncpy(copyto, matches[0], strlen(matches[0]) + 1);
 	    *newplace += strlen(matches[0]) - pos;
 
-	    if (stat(buf, &fileinfo) == -1)
-		break;
-	    else if (S_ISDIR(fileinfo.st_mode)) {
-		strncat(buf, "/", 1);
-		*newplace += 1;
-		/* now we start over again with # of tabs so far */
-		*lastWasTab = 0;
-	    }
+	    /* Is it a directory? */
+	    append_slash_if_dir(buf, lastWasTab, newplace);
+
 	    break;
 	default:
 	    /* Check to see if all matches share a beginning, and if so
@@ -815,3 +1017,4 @@ char *input_tab(char *buf, int place, int *lastWasTab, int *newplace)
     curs_set(1);
     return buf;
 }
+#endif
