@@ -1,4 +1,4 @@
-/* $Id: nano.c,v 1.76 2000/11/21 06:20:19 astyanax Exp $ */
+/* $Id: nano.c,v 1.94 2000/12/03 03:06:45 astyanax Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -104,6 +104,8 @@ RETSIGTYPE finish(int sigage)
 void die(char *msg, ...)
 {
     va_list ap;
+    char *name;
+    int i;
 
     va_start(ap, msg);
     vfprintf(stderr, msg, ap);
@@ -112,12 +114,15 @@ void die(char *msg, ...)
     /* if we can't save we have REAL bad problems,
      * but we might as well TRY. */
     if (filename[0] == '\0') {
-	write_file("nano.save", 0);
+	name = "nano.save";
+	i = write_file(name, 1);
     } else {
-	char buf[BUFSIZ];
-	strncpy(buf, filename, BUFSIZ);
-	strncat(buf, ".save", BUFSIZ - strlen(buf));
-	write_file(buf, 0);
+	
+	char *buf = nmalloc(strlen(filename) + 6);
+	strcpy(buf, filename);
+	strcat(buf, ".save");
+	i = write_file(buf, 1);
+	name = buf;
     }
     /* Restore the old term settings */
     tcsetattr(0, TCSANOW, &oldterm);
@@ -128,7 +133,10 @@ void die(char *msg, ...)
     endwin();
 
     fprintf(stderr, msg);
-    fprintf(stderr, _("\nBuffer written to 'nano.save'\n"));
+    if (i != -1)
+	fprintf(stderr, _("\nBuffer written to %s\n"), name);
+    else
+	fprintf(stderr, _("\nNo %s written (file exists?)\n"), name);
 
     exit(1);			/* We have a problem: exit w/ errorlevel(1) */
 }
@@ -417,7 +425,29 @@ void version(void)
     printf(_(" nano version %s by Chris Allegretta (compiled %s, %s)\n"),
 	   VERSION, __TIME__, __DATE__);
     printf(_
-	   (" Email: nano@nano-editor.org	Web: http://www.nano-editor.org\n"));
+	   (" Email: nano@nano-editor.org	Web: http://www.nano-editor.org"));
+    printf(_("\n Compiled options:"));
+
+#ifdef NANO_SMALL
+    printf(" --enable-tiny");
+#endif
+#ifdef NANO_EXTRA
+    printf(" --enable-extra");
+#endif
+#ifdef DISABLE_TABCOMP
+    printf(" --disable-tabcomp");
+#endif
+#ifdef DISABLE_JUSTIFY
+    printf(" --disable-justify");
+#endif
+#ifdef DISABLE_SPELL
+    printf(" --disable-spell");
+#endif
+#ifdef USE_SLANG
+    printf(" --with-slang");
+#endif
+    printf("\n");
+
 }
 
 filestruct *make_new_node(filestruct * prevnode)
@@ -477,10 +507,19 @@ int no_help(void)
 	return 0;
 }
 
+#ifdef NANO_SMALL
 void nano_small_msg(void)
 {
     statusbar("Sorry, this function not available with nano-tiny option");
 }
+#endif
+
+#if defined(DISABLE_JUSTIFY) || defined(DISABLE_SPELL)
+void nano_disabled_msg(void)
+{
+    statusbar("Sorry, support for this function has been disabled");
+}
+#endif
 
 /* The user typed a printable character; add it to the edit buffer */
 void do_char(char ch)
@@ -579,12 +618,13 @@ int do_enter_void(void)
 
 void do_next_word(void)
 {
-    filestruct *fileptr;
+    filestruct *fileptr, *old;
     int i;
 
     if (current == NULL)
 	return;
 
+    old = current;
     i = current_x;
     for (fileptr = current; fileptr != NULL; fileptr = fileptr->next) {
 	if (fileptr == current) {
@@ -612,8 +652,19 @@ void do_next_word(void)
 
     current_x = i;
     placewewant = xplustabs();
+
     if (current->lineno >= editbot->lineno)
 	edit_update(current, CENTER);
+    else {
+	/* If we've jumped lines, refresh the old line.  We can't just use
+	 * current->prev here, because we may have skipped over some blank
+	 * lines, in which case the previous line is the wrong one.
+	 */
+	if (current != old)
+	    update_line(old, 0);
+
+	update_line(current, current_x);
+    }
 
 }
 
@@ -1051,13 +1102,13 @@ void wrap_reset(void)
     UNSET(SAMELINEWRAP);
 }
 
-#ifndef NANO_SMALL
+#if !defined(NANO_SMALL) && !defined(DISABLE_SPELL)
 
 int do_int_spell_fix(char *word)
 {
     char *prevanswer = NULL, *save_search = NULL, *save_replace = NULL;
-    filestruct *begin, *begin_top;
-    int i = 0, beginx, beginx_top;
+    filestruct *begin;
+    int i = 0, j = 0, beginx, beginx_top;
 
     /* save where we are */
     begin = current;
@@ -1074,25 +1125,32 @@ int do_int_spell_fix(char *word)
     last_replace = mallocstrcpy(last_replace, word);
 
     /* start from the top of file */
-    begin_top = current = fileage;
-    beginx_top = current_x = -1;
+    current = fileage;
+    current_x = beginx_top = -1;
 
     search_last_line = FALSE;
 
+    edit_update(fileage, TOP);
+
     /* make sure word is still mis-spelt (i.e. when multi-errors) */
-    if (findnextstr(TRUE, begin_top, beginx_top, prevanswer) != NULL)
+    if (findnextstr(TRUE, fileage, beginx_top, prevanswer) != NULL)
     {
+	do_replace_highlight(TRUE, prevanswer);
+
+	/* allow replace word to be corrected */
+	i = statusq(0, spell_list, SPELL_LIST_LEN, last_replace, 
+		_("Edit a replacement"));
+
+	do_replace_highlight(FALSE, prevanswer);
+
 	/* start from the start of this line again */
-	current = begin_top;
+	current = fileage;
 	current_x = beginx_top;
 
 	search_last_line = FALSE;
 
-	/* allow replace word to be corrected */
-	i = statusq(0, replace_list_2, REPLACE_LIST_2_LEN, last_replace, 
-		_("Edit a replacement"));
-
-	do_replace_loop(prevanswer, begin_top, &beginx_top, TRUE, &i);
+	j = i;
+	do_replace_loop(prevanswer, fileage, &beginx_top, TRUE, &j);
     }
 
     /* restore the search/replace strings */
@@ -1112,7 +1170,7 @@ int do_int_spell_fix(char *word)
 }
 #endif
 
-#ifndef NANO_SMALL
+#if !defined(NANO_SMALL) && !defined(DISABLE_SPELL)
 
 /* Integrated spell checking using 'spell' program */
 int do_int_speller(char *tempfile_name)
@@ -1153,6 +1211,7 @@ int do_int_speller(char *tempfile_name)
 	}
 	close(tempfile_fd);
 
+
 	/* send spell's standard out to the pipe */
 
 	if (dup2(in_fd[1], STDOUT_FILENO) != STDOUT_FILENO) {
@@ -1241,7 +1300,7 @@ int do_int_speller(char *tempfile_name)
 }
 #endif
 
-#ifndef NANO_SMALL
+#if !defined(NANO_SMALL) && !defined(DISABLE_SPELL)
 
 /* External spell checking */
 int do_alt_speller(char *file_name)
@@ -1293,9 +1352,12 @@ int do_alt_speller(char *file_name)
 int do_spell(void)
 {
 
-#ifdef NANO_SMALL
+#if defined(NANO_SMALL)
     nano_small_msg();
     return (TRUE);
+#elif defined(DISABLE_SPELL)
+    nano_disabled_msg();
+    return (TRUE);
 #else
     char *temp;
     int spell_res;
@@ -1306,8 +1368,10 @@ int do_spell(void)
 	return 0;
     }
 
-    if (write_file(temp, 1) == -1)
+    if (write_file(temp, 1) == -1) {
+	statusbar(_("Spell checking failed: unable to write temp file!"));
 	return 0;
+    }
 
     if (alt_speller)
 	spell_res = do_alt_speller(temp);
@@ -1435,8 +1499,7 @@ void do_mouse(void)
 /* Handler for SIGHUP */
 RETSIGTYPE handle_hup(int signal)
 {
-    write_file("nano.save", 0);
-    finish(1);
+    die(_("Received SIGHUP"));
 }
 
 /* What do we do when we catch the suspend signal */
@@ -1597,7 +1660,7 @@ int do_tab(void)
     return 1;
 }
 
-#ifndef NANO_SMALL
+#if !defined(NANO_SMALL) && !defined(DISABLE_JUSTIFY)
 int empty_line(const char *data)
 {
     while (*data) {
@@ -1649,10 +1712,16 @@ void justify_format(char *data)
 
 int do_justify(void)
 {
-#ifndef NANO_SMALL
+#ifdef NANO_SMALL
+    nano_small_msg();
+    return 1;
+#elif defined(DISABLE_JUSTIFY)
+    nano_disabled_msg();
+    return 1;
+#else
     int slen = 0;		/* length of combined lines on one line. */
-    int initial_y;
-    filestruct *initial = NULL;
+    int initial_y, kbinput;
+    filestruct *initial = NULL, *tmpjust = NULL, *cutbak, *tmptop, *tmpbot;
 
     if (empty_line(current->data)) {
 	/* Justify starting at first non-empty line. */
@@ -1690,6 +1759,12 @@ int do_justify(void)
     initial_y = current_y;
 
     set_modified();
+    cutbak = cutbuffer; /* Got to like cutbak ;) */
+    cutbuffer = NULL;
+
+    tmptop = current;
+    tmpjust = copy_node(current);
+    add_to_cutbuffer(tmpjust);
     /* Put the whole paragraph into one big line. */
     while (current->next && !isspace((int) current->next->data[0])
 	   && current->next->data[0]) {
@@ -1697,6 +1772,10 @@ int do_justify(void)
 	int len = strlen(current->data);
 	int len2 = strlen(current->next->data);
 
+	tmpjust = NULL;
+	tmpjust = copy_node(current->next);
+	add_to_cutbuffer(tmpjust);
+
 	/* length of both strings plus space between strings and ending \0. */
 	current->data = nrealloc(current->data, len + len2 + 2);
 	current->data[len++] = ' ';
@@ -1761,6 +1840,7 @@ int do_justify(void)
 	} while ((strlenpt(current->data) > (fill))
 		 && !no_spaces(current->data));
     }
+    tmpbot = current;
 
     if (current->next)
 	current = current->next;
@@ -1783,10 +1863,37 @@ int do_justify(void)
     }
 
     edit_refresh();
-    statusbar(_("Justify Complete"));
-    return 1;
-#else
-    nano_small_msg();
+    statusbar(_("Can now UnJustify!"));
+    /* Change the shortcut list to display the unjustify code */
+    shortcut_init(1);
+    display_main_list();
+    reset_cursor();
+
+    /* Now get a keystroke and see if it's unjustify, if not unget the keytroke 
+       and return */
+    if ((kbinput = wgetch(edit)) != NANO_UNJUSTIFY_KEY)
+	ungetch(kbinput); 
+    else {
+	/* Else restore the justify we just did (ungrateful user!) */
+	if (tmptop->prev != NULL)
+	    tmptop->prev->next = tmpbot->next;
+	else
+	    fileage = current;
+	tmpbot->next->prev = tmptop->prev;
+ 	current = tmpbot->next;
+	tmpbot->next = NULL;
+	do_uncut_text();
+	if (tmptop->prev == NULL)
+	    edit_refresh();
+
+	free_filestruct(tmptop);
+	blank_statusbar_refresh();
+    }
+    shortcut_init(0);
+    display_main_list();
+    free_filestruct(cutbuffer);
+    cutbuffer = cutbak;
+    
     return 1;
 #endif
 }
@@ -1873,7 +1980,7 @@ void do_toggle(int which)
 
     switch (toggles[which].val) {
     case TOGGLE_PICOMODE_KEY:
-	shortcut_init();
+	shortcut_init(0);
 	display_main_list();
 	break;
     case TOGGLE_SUSPEND_KEY:
@@ -2088,7 +2195,7 @@ int main(int argc, char *argv[])
 
     /* Set up some global variables */
     global_init();
-    shortcut_init();
+    shortcut_init(0);
     init_help_msg();
     help_init();
     signal_init();
@@ -2330,7 +2437,9 @@ int main(int argc, char *argv[])
 #ifdef PDCURSES
 	    case 541:		/* ???? */
 	    case 542:		/* Control and alt in Windows *shrug* */
+	    case 543:		/* Right ctrl key */
 	    case 544:
+	    case 545:		/* Right alt key */
 #endif
 
 		break;
