@@ -1,4 +1,4 @@
-/* $Id: winio.c,v 1.53 2000/11/17 01:37:39 astyanax Exp $ */
+/* $Id: winio.c,v 1.60 2000/11/29 23:43:28 astyanax Exp $ */
 /**************************************************************************
  *   winio.c                                                              *
  *                                                                        *
@@ -22,6 +22,7 @@
 #include <stdarg.h>
 #include <string.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include "config.h"
 #include "proto.h"
 #include "nano.h"
@@ -247,8 +248,11 @@ int nanogetstr(int allowtabs, char *buf, char *def, shortcut s[], int slen,
 	       int start_x)
 {
     int kbinput = 0, j = 0, x = 0, xend;
-    int x_left = 0, inputlen, tabbed = 0, shift = 0;
+    int x_left = 0, inputlen, tabbed = 0;
     char *inputbuf;
+#ifndef DISABLE_TABCOMP
+    int shift = 0;
+#endif
     
     inputbuf = nmalloc(strlen(def) + 1);
     inputbuf[0] = 0;
@@ -286,13 +290,15 @@ int nanogetstr(int allowtabs, char *buf, char *def, shortcut s[], int slen,
 	case 343:
 	    ungetch(13);	/* Enter on iris-ansi $TERM, sometimes */
 	    break;
+	/* Stuff we want to ignore */
 #ifdef PDCURSES
-	/* Stuff we want to ignore with PDCURSES */
 	case 541:
 	case 542:
+	case 543:			/* Right ctrl again */
 	case 544:
-		break;
+	case 545: 			/* Right alt again */
 #endif
+		break;
 	case KEY_HOME:
 	    x = x_left;
 	    nanoget_repaint(buf, inputbuf, x);
@@ -340,6 +346,7 @@ int nanogetstr(int allowtabs, char *buf, char *def, shortcut s[], int slen,
 		x--;
 	    nanoget_repaint(buf, inputbuf, x);
 	    break;
+#ifndef DISABLE_TABCOMP
 	case NANO_CONTROL_I:
 	    if (allowtabs) {
 		shift = 0;
@@ -349,6 +356,7 @@ int nanogetstr(int allowtabs, char *buf, char *def, shortcut s[], int slen,
 		nanoget_repaint(buf, inputbuf, x);
 	    }
 	    break;
+#endif
 	case KEY_LEFT:
 	    if (x > strlen(buf))
 		x--;
@@ -1322,3 +1330,102 @@ void fix_editbot(void)
     for (i = 0; (i <= editwinrows - 1) && (editbot->next != NULL)
 	 && (editbot != filebot); i++, editbot = editbot->next);
 }
+
+#ifdef NANO_EXTRA
+#define CREDIT_LEN 45
+void do_credits(void)
+{
+    int i, j = 0, k, place = 0, start_x;
+    char *what;
+
+    char *nanotext = _("The nano text editor");
+    char *version = _("version ");
+    char *brought = _("Brought to you by:");
+    char *specialthx = _("Special thanks to:");
+    char *fsf = _("The Free Software Foundation");
+    char *ncurses = _("Pavel Curtis, Zeyd Ben-Halim and Eric S. Raymond for ncurses");
+    char *anyonelse = _("and anyone else we forgot...");
+    char *thankyou = _("Thank you for using nano!\n");
+
+    char *credits[CREDIT_LEN] = {nanotext, 
+			version, 
+			VERSION, 
+			"",
+			brought,
+			"Chris Allegretta",
+			"Jordi Mallach",
+			"Adam Rogoyski",
+			"Rob Siemborski",
+			"Rocco Corsi",
+			"Ken Tyler",
+			"Sven Guckes",
+			"Florian Knig",
+			"Pauli Virtanen",
+			"Daniele Medri",
+			"Clement Laforet",
+			"Tedi Heriyanto",
+			"Bill Soudan",
+			"Christian Weisgerber",
+			"Erik Anderson",
+			"Big Gaute",
+			"Joshua Jensen",
+			"",
+			specialthx,
+			"Plattsburgh State University",
+			"Benet Laboratories",
+			"Amy Allegretta",
+			"Linda Young",
+			"Jeremy Robichaud",
+			"Richard Kolb II",
+			fsf,
+			"Linus Torvalds",
+			ncurses,
+			anyonelse,
+			thankyou,
+			"", "", "", "",
+			"(c) 2000 Chris Allegretta",
+			"", "", "", "",
+			"www.nano-editor.org"
+    };
+
+    curs_set(0);
+    nodelay(edit, TRUE);
+    blank_bottombars();
+    mvwaddstr(topwin, 0, 0, hblank);
+    blank_edit();
+    wrefresh(edit);
+    wrefresh(bottomwin);
+    wrefresh(topwin);
+
+    while (wgetch(edit) == ERR) {
+	for (k = 0; k <= 1; k++) {
+	    blank_edit();
+	    for (i = editwinrows / 2 - 1; i >= (editwinrows / 2 - 1 - j); i--) {
+		mvwaddstr(edit, i * 2 - k, 0, hblank);
+
+		if (place - (editwinrows / 2 - 1 - i) < CREDIT_LEN)
+		    what = credits[place - (editwinrows / 2 - 1 - i)];
+		else
+		    what = "";
+
+		start_x = center_x - strlen(what) / 2 - 1;
+		mvwaddstr(edit, i * 2 - k, start_x, what);
+	    }
+	    usleep(700000);
+	    wrefresh(edit);
+	}
+	if (j < editwinrows / 2 - 1)
+	    j++;
+
+	place++;
+
+	if (place >= CREDIT_LEN + editwinrows / 2)
+	    break;
+    }
+
+    nodelay(edit, FALSE);
+    curs_set(1);
+    display_main_list();
+    total_refresh();
+ }
+#endif
